#"@(#) SGeSAP/LX - 12.10.00 - $Rev: 1651 $ - Last modified 02/02/17"
#
# **********************************************************************
# *                                                                    *
# *  HPE SGeSAP/LX                                                     *
# *                                                                    *
# *  SAP Customer Function Definition File                             *
# *                                                                    *
# *  (c) Copyright 1996-2015 Hewlett-Packard Co.                       *
# *  (c) Copyright 2015-2017 Hewlett Packard Enterprise Co.            *
# *                                                                    *
# *                                                                    *
# * Description: SGeSAP function depot definition template             *
# *             file for customer specific functions.                  *
# *                                                                    *
# * Note:       This function depot file consists of contributed       *
# *	        examples of additional SGeSAP script functions.        *
# *             The functionality is hooked into SGeSAP execution      *
# *             logic.                                                 *
# *                                                                    *
# *             After copying the file to                              *
# *             ${SGCONF}/ext/${SG_PACKAGE_NAME}_customer_functions.sh *
# *             it might be edited to customize behavior.              *
# *                                                                    *
# *             The file needs to be distributed manually to all       *
# *             nodes in the cluster after changing it.                *
# *                                                                    *
# *             HPE support might ask to copy further standard         *
# *             SGeSAP control script functions into the file to       *
# *             apply temporary quick fixes.                           *
# *                                                                    *
# *             The original template file must not be edited.         *
# *                                                                    *
# **********************************************************************


########################################################################
#
# Function: notify_bia
#
# This function is a framework that allows to hook in a
# switchover.py script to notify a BIA system as part of
# a BI/BW failover procedure.
# This functionality is not complete without a switchover.py
# python script.
# There is currently no official notification script available.
# SAP might decide to ship one as part of BIA in the future.
#
# Parameters:
# $1      SAP SID
#
########################################################################
function notify_bia
{
  typeset fname=notify_bia
  BLogMsg $*
 
  typeset host
  typeset biamaster
  typeset -i i

  LogMsg "Triggering BIA operation ..."
  biamaster=1
  i=0
  for host in ${BIASTATIC[@]}
  do
    (( i = $i + 1 ))
    is_node_alive ${host}&
    watchdog $!
    if [[ $? -eq 1 ]]; then
      LogMsg2 2 "BIA Nameserver ${i} on ${host} reachable"
      check_login ${host} ${BIAADM}&
      watchdog $!
      if [[ $? -eq 0 ]]; then
        if [[ ${biamaster} -eq 1 ]]; then
          (${REM_COMM} ${host} -l ${BIAADM} -n "${BIAPATH}${SWITCHOVERPY} --switch --debug=${SG_LOG_LEVEL}" 2>&1|logpipe|
            awk '/ GREEN Result=/  {green=1; continue}
                 / YELLOW Result=/ {green=1; continue}
                 / RED Result=/    {red=1; continue}
                 /./          {print $0}
                 END {if (red) exit 1; if (!green) exit 2}')&
          watchdog $!
          case $? in
          0)
            biamaster=${host}
          ;;
          1)
            WrnLogMsg "BIA operation reports failure on ${host}"
          ;;
          2)
            WrnLogMsg "BIA operation failed on ${host}"
          esac
        fi
      fi
    else
      WrnLogMsg "BIA Nameserver ${i} on ${host} not reachable"
    fi
  done
  if [[ ${biamaster} -eq 1 ]]; then
    WrnLogMsg "BIA operation failed"
  else
    LogMsg "BIA operation reports success on ${biamaster}"
  fi
}

