#"@(#) SGeSAP/LX - 12.10.00 - $Rev: 773 $ - Last modified 02/02/17"
#
# **********************************************************************
# *                                                                    *
# * HPE SGeSAP/LX                                                      *
# *                                                                    *
# * SAP Netweaver module enqor gatekeeper script                       *
# *                                                                    *
# *  (c) Copyright 1996-2015 Hewlett-Packard Co.                       *
# *  (c) Copyright 2015-2017 Hewlett Packard Enterprise Co.            *
# *                                                                    *
# *                                                                    *
# * Description: Serviceguard Extension for SAP control logic          *
# *                                                                    *
# * This control script file must not be edited !                      *
# * Any changes made to it will be overwritten when you upgrade to the *
# * next revision of SGeSAP.                                           *
# *                                                                    *
# * Changing this file may lead to unrecoverable package startup       *
# * and/or shutdown problems.                                          *
# *                                                                    *
# **********************************************************************


. ${SGCONF}/scripts/sgesap/sap_control.sh

set -m
trap "wait;sg_log 2 \"Interrupting SGeSAP ${script_name} '${op_code}' runtime steps ...\"; exit 2" 2

case ${op_code} in
  start)
    echo ${SGeSAP_SAP_INSTANCE[@]}|grep -e SCS -e ERS >/dev/null
    if [ $? -eq 0 ]; then
      #
      # wake resource monitors because of package start op
      #
      LogMsg "Signal enqueue operating resource monitors"
      ${SGSBIN}/cmdo -t ${WATCHDOG_TIMEOUT} 'TPID=`ps -eo pid,cmd|grep -v grep|grep /bin/sh|grep /monitors/sgesap/sapenqor.mon|awk '\''{ print $1 }'\''`;
        if [ "${TPID}" != "0" -a -n "${TPID}" ]; then
          kill -USR1 ${TPID};
          echo Signal sapenqor.mon [${TPID}] on $(hostname);
        fi;'|logpipe
      sleep 3
      i=0
      while [ -f "${ENQORMON_SYNCFLAG}" -a ${DELAY_INTERVALS} -gt ${i} ];
      do
        test_proc sapenqor.mon
        if [ $? -ne 1 ]; then
          LogMsg2 5 "Wait for sapenqor.mon to refresh resource values"
          sleep ${GATEKEEPER_SLEEP:-15}
          (( i = ${i} + 1 ))
        else
          WrnLogMsg "sapenqor.mon not running, but orphaned syncflag exists"
          WrnLogMsg "sapenqor.mon might have been crashed previously"
          LogMsg "Removing syncflag"
          rm ${ENQORMON_SYNCFLAG}
          break
        fi
      done

      #
      # the following code is a safety net to catch errors of malfunctioning enqor resources
      # (this should only happen if someone manually changes resource values via CLI and a
      #  failover occurs before SGeSAP can correct the value)
      #
      # The replication packages doublecheck the state of the package(s) from which they replicate
      # and decide based on that whether to continue, wait or fail.
      #
      if [ -n "${SGeSAP_SAP_REPLICATED_INSTANCE[0]}" ]; then
        for pkg in `${SGSBIN}/cmviewcl -s config -l package -f line 2>/dev/null|grep "^package:[[:alnum:]_-]*|name="|cut -f 2 -d =`; do
          skippkg=0
          for line in `${SGSBIN}/cmviewcl -f line -p ${pkg} -v`; do
            if [ ${skippkg} -eq 1 ]; then
              break
            fi
            if [ `echo ${line}|cut -f 1 -d =|grep sgesap/stack/sap_instance|grep SCS|wc -l` -ne 0 ]; then
              scs_tmp=`echo ${line}|cut -f 2 -d =`
              for inst in ${SGeSAP_SAP_REPLICATED_INSTANCE[@]}
              do
                if [ ${skippkg} -eq 1 ]; then
                  break
                fi
                if [ ${inst} = ${scs_tmp} ]; then
                  if [ `${SGSBIN}/cmviewcl -p ${pkg} -f line|egrep "^owner=(${HNAME}|unknown)$"|wc -l` -ne 0 ]; then
                    WrnLogMsg "Package ${pkg} with ${inst} is owned by the local node or ownership is unknown - wait..."
                    #
                    # enqueue is here, we might be in trouble
                    # wait to see whether it decides to stay
                    #
                    while true
                    do
                      state=`${SGSBIN}/cmviewcl -p ${pkg} -f line|grep state=`
                      if [ `${SGSBIN}/cmviewcl -p ${pkg} -f line|egrep "^owner=(${HNAME}|unknown)$"|wc -l` -ne 0 ]; then
                        LogMsg "Package ${pkg} with ${inst} no longer owned by local node - continue..."
                        skippkg=1
                        break
                      fi
                      if [ `echo ${state}|grep -e maintenance -e running -e halt_aborted -e detached|wc -l` -ne 0 ]; then
                        ErrLogMsg "Package ${pkg} with ${inst} is in stable ${state} - replication is not allowed locally"
                        sleep 5
                        exit ${ERROR}
                      fi
                      sleep 5
                    done
                  fi
                fi
              done
            fi
          done
        done
      fi
    fi
  ;;
  stop)
    echo ${SGeSAP_SAP_INSTANCE[@]}|grep -e SCS -e ERS >/dev/null
    if [ $? -eq 0 ]; then
      #
      # wake resource monitors because of package stop op
      #
      LogMsg "Signal enqueue operating resource monitors"
      ${SGSBIN}/cmdo -t ${WATCHDOG_TIMEOUT} 'TPID=`ps -eo pid,cmd|grep -v grep|grep "/bin/sh '${SGCONF}'/monitors/sgesap/sapenqor.mon"|awk '\''{ print $1 }'\''`;
        if [ "${TPID}" != "0" -a -n "${TPID}" ]; then
          kill -USR1 ${TPID};
          echo Signal sapenqor.mon [${TPID}] on $(hostname);
        fi;'|logpipe
    fi
  ;;
  validate)
    #
    # wake resource monitor because of potential
    # upcoming configuration change
    #
    TPID=`ps -eo pid,cmd|grep -v grep|grep "/bin/sh ${SGCONF}/monitors/sgesap/sapenqor.mon"|awk '{ print $1 }'`;
    if [ "${TPID}" != "0" -a -n "${TPID}" ]; then
      kill -USR2 ${TPID}
      LogMsg2 5 "Signal sapenqor.mon [${TPID}] on local host"
    fi
    SAPVERIFY="${SGSBIN}/sapverify"
    if [ -x "${SAPVERIFY}" ]; then
       ${SAPVERIFY} -p "${SG_PACKAGE}" -sg_op_code "${op_code}" -call "$0"
       rc=$?
       [ ${rc} -gt 0 ] && exit ${rc}
    else
        sg_log 0 "WARNING: Couldn't find or execute ${SAPVERIFY}"
    fi

  ;;
  reconfigure_start)
  ;;
  reconfigure_stop)
  ;;
  *)
    sg_log 0 "ERROR: Runtime called with illegal package mode: '${op_code}'"
    exit 2
  ;;
esac

sg_log 2 "Leaving SGeSAP ${script_name} '${op_code}' runtime steps ..."

#return 0
