#"@(#) SGeSAP/LX - 12.10.00 - $Rev: 1185 $ - Last modified 02/02/17"
#
# **********************************************************************
# *                                                                    *
# * HPE SGeSAP/LX                                                      *
# *                                                                    *
# * SAP HANA instance handling script                                  *
# *                                                                    *
# * (c) Copyright 1996-2015 Hewlett-Packard Co.                        *
# * (c) Copyright 2015-2017 Hewlett Packard Enterprise Co.             *
# *                                                                    *
# *                                                                    *
# * Description: Serviceguard Extension for SAP control logic          *
# *                                                                    *
# * This control script file must not be edited !                      *
# * Any changes made to it will be overwritten when you upgrade to the *
# * next revision of HPE SGeSAP.                                       *
# *                                                                    *
# * Changing this file may lead to unrecoverable package startup       *
# * and/or shutdown problems.                                          *
# *                                                                    *
# **********************************************************************


if [ -z ${SGCONF} ]; then
  [ -f /etc/cmcluster.conf ] && . /etc/cmcluster.conf
fi

[ -f ${SGCONF}/scripts/sgesap/sap_control.sh ] && . ${SGCONF}/scripts/sgesap/sap_control.sh
fname=hdbdistinstance.sh

set -m
trap "wait;sg_log 2 \"Interrupting SGeSAP ${script_name} '${op_code}' runtime steps ...\"; exit 2" 2

case ${op_code} in
  start)
    check_versions
    # check_debug
    detect_module sgesap/hdbprimary || 
    clean_syncgate ${SGeSAP_HDB_INSTANCE[0]}
    hdb_ipconf_validate|| exit ${FATAL}
    init_params_hdb_inst
    select_ips ${SGeSAP_HDB_INSTANCE[0]}
    if [ $? -ne 0 ]; then
      LogMsg "Wait and make a second attempt after 5 minutes"
      sleep 300
      select_ips ${SGeSAP_HDB_INSTANCE[0]}||exit ${ERROR}
    fi
    if [ "X${GLOBINIUPD[0]}" != "X" ]; then # found at least one ip
      if [ ${SGeSAP_HDB_UPDATE_PUBLIC_RESOLUTION} -eq 1 ]; then
        flock -x 199
        backup_globalini ${GLOBALINI}
        cleanup_public_resolution ${GLOBALINI}
        if [ $? -ne 0 ]; then
          set_globalini_dp_param ${GLOBALINI} 
        fi
      fi 199<${GLOBALINI}
    fi
    [ ${SG_LOG_LEVEL:-0} -gt 3 ] && 
      LogMsg "HANA Public Resolution: map_${HOSTNAME}=`read_ini_config ${GLOBALINI} public_hostname_resolution map_${HOSTNAME}`"
    . ${SGCONF}/scripts/sg/package_ip.sh start
  ;;
  stop)
    check_versions
    select_ips ${SGeSAP_HDB_INSTANCE[0]}
    . ${SGCONF}/scripts/sg/package_ip.sh stop
  ;;
  validate)
    hdb_ipconf_validate
    exit $?
  ;;
  reconfigure_start)
    hdb_ipconf_validate
    static_param_latch HDB_IP_SUBNET
    static_param_latch HDB_IP_ADDRESS
    exit $?
  ;;
  reconfigure_stop)
  ;;
  *)
    sg_log 0 "ERROR: Runtime called with illegal package mode: '${op_code}'"
    exit 2
  ;;
esac

sg_log 2 "Leaving SGeSAP ${script_name} '${op_code}' runtime steps ..."

exit 0
