#"@(#) SGeSAP/LX - 12.10.00 - $Rev: 1657 $ - Last modified 02/02/17"
#
# **********************************************************************
# *                                                                    *
# * HPE SGeSAP/LX                                                      *
# *                                                                    *
# * SAP HANA instance handling script                                  *
# *                                                                    *
# * (c) Copyright 1996-2015 Hewlett-Packard Co.                        *
# * (c) Copyright 2015-2016 Hewlett Packard Enterprise Co.             *
# *                                                                    *
# *                                                                    *
# * Description: Serviceguard Extension for SAP control logic          *
# *                                                                    *
# * This control script file must not be edited !                      *
# * Any changes made to it will be overwritten when you upgrade to the *
# * next revision of HPE SGeSAP.                                       *
# *                                                                    *
# * Changing this file may lead to unrecoverable package startup       *
# * and/or shutdown problems.                                          *
# *                                                                    *
# **********************************************************************

if [ -z ${SGCONF} ]; then
  [ -f /etc/cmcluster.conf ] && . /etc/cmcluster.conf
fi

[ -f ${SGCONF}/scripts/sgesap/sap_control.sh ] && . ${SGCONF}/scripts/sgesap/sap_control.sh
fname=hdbinstance.sh

set -m
trap "wait;sg_log 2 \"Interrupting SGeSAP ${script_name} '${op_code}' runtime steps ...\"; safesync_listener off; exit 2" INT

case ${op_code} in
  start)
    check_versions
    check_debug 'detect_module_hdbprimary; set_smart_quorum_prio ${HDBSECONDARY}' # actions before exit if pkg dbg flag
    init_params_hdb_inst

    if [ ${HDB_T} ]; then

      HDBNAME=`echo ${SGeSAP_HDB_INSTANCE}|eval ${GETNAME}`
      HDBNR=`echo ${SGeSAP_HDB_INSTANCE}|eval ${GETNR}`
      set_environment ${HDBNAME} ${HDBNR} ${VHNAME} ${SGeSAP_HDB_SYSTEM}
      check_safesync

      if [ ${HDBSECONDARY} -eq 1 ]; then
        # for primary this was already done in the primary module
        clean_syncgate
        prepare_sapserviceconfig
      else
        trap "safesync_listener off" EXIT
        safesync_listener on
      fi

      stop_sapstartsrv ${HDBNAME} ${HDBNR} ${VHNAME} Linux ${MYSIDADM} ${SGeSAP_HDB_SYSTEM}
      clean_ipc ${HDBNAME} ${HDBNR} ${MYSIDADM}
      start_sapstartsrv ${HDBNAME} ${HDBNR} ${VHNAME} ${SGeSAP_HDB_SYSTEM}

      if [ ${HDBSCALEOUT} -eq 1 ]; then
        if [ ${HDBSECONDARY} -eq 1 ]; then
          hdb_syncgate ${SGeSAP_HDB_INSTANCE} s
        else
          safesync_listener_candidate ${SGeSAP_HDB_INSTANCE}
          hdb_syncgate ${SGeSAP_HDB_INSTANCE} n
        fi
        if [ ${?} -ne 0 ]; then
          exit ${FATAL}
        fi
      else
        check_replication ${HDBNAME} ${HDBNR} ${SGeSAP_HDB_SYSTEM}
        # 1) check_replication returns this is not a secondary instance
        # e.g. global.ini identifies this is a primary instance
        # 2) package itself doesn't have a primary module configured:
        # => demotion phase: secondary package is about to be started
        # on a previous primary. sanity check in decide_role(): demotion
        # will only be allowed if primary package is in a running
        # state so it is not demoted by accident
        if [ ${?} -ne 2 ]; then # system can not be identified as secondary
          if [ ${HDBSECONDARY} -eq 1 ]; then
            decide_role ${HDBNAME} ${HDBNR} ${SGeSAP_HDB_SYSTEM}||exit ${?}
          fi
        fi

        set_smart_quorum_prio ${HDBSECONDARY}

        # if dual-purpose config update global.ini parameters.
        # adjustment for primary already executed from hdbprimary
        # module, so just update secondary here
        if [[ ${HDBSECONDARY} -eq 1 && ${#SGeSAP_HDB_EPS_LOCATION[@]} -gt 0 ]]; then
          update_globalini
        fi
      fi

      start_hdb ${HDBNAME} ${HDBNR} ${SGeSAP_HDB_SYSTEM}

      # log completion of primary startup
      register_productionstart

      # safesync_listener off

    fi
  ;;
  stop)
    check_versions
    init_params_hdb_inst
    check_safesync

    if [[ ${HDBSECONDARY} -eq 0 || ${SGeSAP_HDB_STOPWITHPKG} -eq 1 ]]; then
      # stop only if primary
      # WVE: and in failover scenario: so don't move this code to primary module

      init_params_hdb_inst

      trap "safesync_listener off" EXIT
      safesync_listener on

      # if configured enable quiesce phase (e.g. quiesce DB connections)
      # run quiesce only on primary e.g. HDBSECONDARY=0
      if [[ (${#SGeSAP_HDB_QUIESCE_SYSTEM[@]} -gt 0 ) && ( $HDBSECONDARY -eq 0 ) ]]; then
            set_quiesce_phase enable
      fi

      if [ ${HDBSCALEOUT} -ne 1 ]; then
        set_smart_quorum_prio 3
      fi
      if  [ ${HDB_T} ]; then
        HDBNAME=`echo ${SGeSAP_HDB_INSTANCE}|eval ${GETNAME}`
        HDBNR=`echo ${SGeSAP_HDB_INSTANCE}|eval ${GETNR}`
        set_environment ${HDBNAME} ${HDBNR} ${VHNAME} ${SGeSAP_HDB_SYSTEM}
        prepare_sapserviceconfig
        stop_hdb ${HDBNAME} ${HDBNR} ${SGeSAP_HDB_SYSTEM}
        stop_sapstartsrv ${HDBNAME} ${HDBNR} ${VHNAME} Linux ${MYSIDADM} ${SGeSAP_HDB_SYSTEM}
      fi
    else
      LogMsg2 4 "Skip any halt operation for instances of secondary systems."
    fi
  ;;
  validate)
    check_versions
    init_params_hdb_inst

    SAPVERIFY="${SGSBIN}/sapverify"
    if [ -x "${SAPVERIFY}" ]; then
       ${SAPVERIFY} -p "${SG_PACKAGE}" -sg_op_code "${op_code}" -call "$0"
       rc=$?
       [ ${rc} -gt 0 ] && exit ${rc}
    else
        sg_log 0 "WARNING: Couldn't find or execute ${SAPVERIFY}"
    fi


  ;;
  reconfigure_start)
  ;;
  reconfigure_stop)
  ;;
  *)
    sg_log 0 "ERROR: Runtime called with illegal package mode: '${op_code}'"
    exit 2
  ;;
esac

sg_log 2 "Leaving SGeSAP ${script_name} '${op_code}' runtime steps ..."

exit 0
