#"@(#) SGeSAP/LX - 12.10.00 - $Rev: 1653 $ - Last modified 02/02/17"
#
# **********************************************************************
# *                                                                    *
# * HPE SGeSAP/LX                                                      *
# *                                                                    *
# * SAP HANA primary token handling script                             *
# *                                                                    *
# * (c) Copyright 1996-2015 Hewlett-Packard Co.                        *
# * (c) Copyright 2015-2016 Hewlett Packard Enterprise Co.             *
# *                                                                    *
# *                                                                    *
# * Description: Serviceguard Extension for SAP control logic          *
# *                                                                    *
# * This control script file must not be edited !                      *
# * Any changes made to it will be overwritten when you upgrade to the *
# * next revision of HPE SGeSAP.                                       *
# *                                                                    *
# * Changing this file may lead to unrecoverable package startup       *
# * and/or shutdown problems.                                          *
# *                                                                    *
# **********************************************************************

if [ -z ${SGCONF} ]; then
  [ -f /etc/cmcluster.conf ] && . /etc/cmcluster.conf
fi

[ -f ${SGCONF}/scripts/sgesap/sap_control.sh ] && . ${SGCONF}/scripts/sgesap/sap_control.sh

set -m
trap "wait;sg_log 2 \"Interrupting SGeSAP ${script_name} '${op_code}' runtime steps ...\"; safesync_listener off; exit 2" INT
 
case ${op_code} in
  start)
    check_versions
    check_debug
    init_params_hdb_inst

    if [ ${HDB_T} ]; then
      prepare_sapserviceconfig
      HDBNAME=`echo ${SGeSAP_HDB_INSTANCE}|eval ${GETNAME}`
      HDBNR=`echo ${SGeSAP_HDB_INSTANCE}|eval ${GETNR}`
      set_environment ${HDBNAME} ${HDBNR} ${VHNAME} ${SGeSAP_HDB_SYSTEM}
      check_safesync
      if [ ${HDBSCALEOUT} -eq 1 ]; then
        clean_syncgate ${SGeSAP_HDB_INSTANCE}
      else
        # execute the scale-up syncgate early, to speed up execution of package start
        safesync_init
        trap "safesync_listener off" EXIT
        safesync_listener on  # needed already by takeover operations
      fi
      find_sap_binary ${HDBNAME} ${HDBNR} ${VHNAME} ${SGeSAP_HDB_SYSTEM} sappfpar
      CMD="${LOCATION:-sappfpar} pf=${MYINSTPROFILE} Autostart"
      if [[ "$(execute_sapcmd_sLPATH root "$CMD" | tail -1)" = "1" ]]; then
        WrnLogMsg "Instance Autostart via Instance Agent detected - delay start of Instance Agent"
      else
        stop_sapstartsrv ${HDBNAME} ${HDBNR} ${VHNAME} Linux ${MYSIDADM} ${SGeSAP_HDB_SYSTEM}
        clean_ipc ${HDBNAME} ${HDBNR} ${MYSIDADM}
        start_sapstartsrv ${HDBNAME} ${HDBNR} ${VHNAME} ${SGeSAP_HDB_SYSTEM}
      fi
      if [ ${HDBSCALEOUT} -eq 1 ]; then
        safesync_listener_candidate ${SGeSAP_HDB_INSTANCE}
        hdb_syncgate ${SGeSAP_HDB_INSTANCE} p  # calls safesync_init and takeover_hdb
      else
        takeover_hdb ${HDBNAME} ${HDBNR} ${SGeSAP_HDB_SYSTEM}||exit $?
      fi
    fi

  ;;
  stop)
    # nothing to do
    :
  ;;
  validate)
    check_versions
    init_params_hdb_inst

    SAPVERIFY="${SGSBIN}/sapverify"
    if [ -x "${SAPVERIFY}" ]; then
       ${SAPVERIFY} -p "${SG_PACKAGE}" -sg_op_code "${op_code}" -call "$0"
       rc=$?
       [ ${rc} -gt 0 ] && exit ${rc}
    else
        sg_log 0 "WARNING: Couldn't find or execute ${SAPVERIFY}"
    fi

  ;;
  reconfigure_start)
  ;;
  reconfigure_stop)
  ;;
  *)
    sg_log 0 "ERROR: Runtime called with illegal package mode: '${op_code}'"
    exit 2
  ;;
esac

sg_log 2 "Leaving SGeSAP ${script_name} '${op_code}' runtime steps ..."

exit 0
