#"@(#) SGeSAP/LX - 12.10.00 - $Rev: 1651 $ - Last modified 02/02/17"
#
# **********************************************************************
# *                                                                    *
# * HPE SGeSAP/LX                                                      *
# *                                                                    *
# * Serviceguard Extension for SAP - Function Definition File          *
# *                                                                    *
# * (c) Copyright 1996-2015 Hewlett-Packard Co.                        *
# * (c) Copyright 2015-2017 Hewlett Packard Enterprise Co.             *
# *                                                                    *
# *                                                                    *
# * Description: Serviceguard Extension for SAP control logic          *
# *                                                                    *
# * This control script file must not be edited !                      *
# * Any changes made to it will be overwritten when you upgrade to the *
# * next revision of HPE SGeSAP.                                       *
# *                                                                    *
# * Changing this file may lead to unrecoverable package startup       *
# * and/or shutdown problems.                                          *
# *                                                                    *
# **********************************************************************


#########################################################################
#
# Function: m_check_if_stopped
#
# Checks if a SAP instance was manually stopped while the package was 
# running e.g. a "sapcontrol -nr xx -function Stop" command was run 
# outside of SGeSAP. If so the SAP_HA_API creates a flag/directory 
#   ${APIDIR}/sgesap_${SAPSYSTEMNAME}_${1}${2}_stopped
#
# Parameters:
#
# $1 INSTNAME
# $2 INSTNR
# $3 last result
#
# Return codes:
#
# 5-7  instance stop was manually triggered
# 4    instance should be stopped
# 1-3  instance start was manually triggered
# 0    instance should be running
#
#########################################################################
function m_check_if_stopped
{
  typeset fname=m_check_if_stopped

#  DbgLogMsg "SAPSYSTEMNAME=${SAPSYSTEMNAME} instance=[${1}][${2}] LASTRESULT=[${3}]"

  typeset rc

  if [ -d ${APIDIR}/sgesap_${SAPSYSTEMNAME}_${1}${2}_stopped ]; then

    DbgLogMsg "Found flag/directory ${APIDIR}/sgesap_${SAPSYSTEMNAME}_${1}${2}_stopped"

    if [ ${3} -lt 4 ]; then
      LogMsg "Manual stop operation detected for ${1}${2}"
      DbgLogMsg "Sleep 20"
      sleep 20
      return 7
    fi
    if [ ${3} -eq 4 ]; then
      return 4
    fi
  else

    DbgLogMsg "No flag/directory ${APIDIR}/sgesap_${SAPSYSTEMNAME}_${1}${2}_stopped"

    if [ ${3} -ge 4 ]; then
      LogMsg "Manual start operation detected for ${1}${2}"
      DbgLogMsg "Sleep 20"
      sleep 20
      return 3
    fi
    if [ ${3} -eq 1 ]; then
      LogMsg2 2 "Resume monitor operation of ${1}${2}"
    fi
    if [ ${3} -eq 0 ]; then
      return 0
    fi
  fi
  (( rc = $3 - 1 ))
  return $rc
}

########################################################################
# init the state machine
#
# Parameters:
#   
# None
#
########################################################################
function m_init_states
{
  typeset fname=m_init_states
  [ ${DBG} -gt 1 ] && BLogMsg $*

    i=0
    while (( $i < ${#SGeSAP_SAP_INSTANCE[@]} ))
    do
      STATUS_FLAG[$i]=GREEN
      YELLOW_COUNT[$i]=0
      STOP_COUNT[$i]=0
      (( i = i + 1 ))
    done
}
########################################################################
# update external/manual stop states
#
# Parameters:
#
# $1 INSTNAME
# $2 INSTNR
# $3 i: array index of all instances in a SG package
#
########################################################################
function m_set_external_states 
{
  typeset fname=m_set_external_states
  [ ${DBG} -gt 1 ] && BLogMsg $*

  typeset INAME=$1
  typeset INR=$2
  typeset i=$3

  alert_status_new=0
  alert_changed=0
  alert_params_new[${PKG_SGESAP_SERVICE_RED}]=""
  alert_params_new[${PKG_SGESAP_SERVICE_YELLOW}]=""
  alert_params_new[${PKG_SGESAP_MANUALLY_HALTED}]=""

  # Init manual start/stopped state of an instance
  if [ "X${WAIT_STOP[$i]}" != "X1" -a "X${WAIT_STOP[$i]}" != "X2" ]; then
    WAIT_STOP[$i]=0
  fi
  # Check if a SAP instance was manually stopped 
  # while the package was running under SGeSAP 
  # control e.g. check for flag directory
  # ${APIDIR}/sgesap_${SAPSYSTEMNAME}_${1}${2}_stopped
  #
  m_check_if_stopped ${INAME} ${INR} ${STOP_COUNT[$i]}
  STOP_COUNT[$i]=$?

  # it was manually stopped 
  if [ ${STOP_COUNT[$i]} -eq 7 ]; then
    WAIT_STOP[$i]=1
#    DbgLogMsg "Manual stop was triggered: STOP_COUNT[$i]=[${STOP_COUNT[$i]}]: Setting WAIT_STOP[$i]=[${WAIT_STOP[$i]}]"
  fi
  DbgLogMsg "i=[${i}] STOP_COUNT[$i]=[${STOP_COUNT[$i]}] WAIT_STOP[$i]=[${WAIT_STOP[$i]}] STATUS_FLAG[$i]=[${STATUS_FLAG[$i]}]"

  if [ ${STOP_COUNT[$i]} -ne 0 ]; then
    (( alert_status_new = ${alert_status_new} | ${PKG_SGESAP_MANUALLY_HALTED} ))
    alert_params_new[$PKG_SGESAP_MANUALLY_HALTED]="${alert_params_new[$PKG_SGESAP_MANUALLY_HALTED]}${alert_params_new[$PKG_SGESAP_MANUALLY_HALTED]:+,}${INAME}${INR}"
  fi

}
########################################################################
# update external/manual stop counter
#
# Parameters:
#
# $1 INSTNAME
# $2 INSTNR
# $3 ALIVE: FLAG if instance is alive (1=yes, 0=no)
# $4 i: array index of all instances in a SG package
#
########################################################################
function m_update_stopcounter
{
  typeset fname=m_update_stopcounter
  [ ${DBG} -gt 1 ] && BLogMsg $*

  typeset INAME=$1
  typeset INR=$2
  typeset ALIVE=$3
  typeset i=$4

  # manual stop was issued
  if [ ${WAIT_STOP[$i]} -gt 0 ];then
    # instance is stopped
    if [ ${ALIVE} -eq 0 ]; then
      WAIT_STOP[$i]=0
      STOP_COUNT[$i]=4
      LogMsg "Manual stop in effect for ${INAME}${INR}"
    fi
  fi

  DbgLogMsg "i=[$i] ALIVE=[${ALIVE}] WAIT_STOP[$i]=[${WAIT_STOP[$i]}] STOP_COUNT[$i]=[${STOP_COUNT[$i]}]"
}

########################################################################
# doublecheck external/manual stop state vs SAP autostart
#
# Parameters:
#
# $1 INSTNAME
# $2 INSTNR
# $3 ALIVE: FLAG if instance is alive (1=yes, 0=no)
# $4 i: array index for all instances in a SG package
#
########################################################################
function m_check_autostart
{
  typeset fname=m_check_autostart
  [ ${DBG} -gt 1 ] && BLogMsg $*

  typeset INAME=$1
  typeset INR=$2
  typeset ALIVE=$3
  typeset i=$4

  DbgLogMsg "ALIVE=[${ALIVE}] i=[$i] WAIT_STOP=[${WAIT_STOP[$i]}] STOP_COUNT=[${STOP_COUNT[$i]}] STATUS_FLAG=[${STATUS_FLAG[$i]}] YELLOW_COUNT=[${YELLOW_COUNT[$i]}]"

  if [ ${STOP_COUNT[$i]} -ne 0 ]; then
    STATUS_FLAG[$i]=GREEN
    YELLOW_COUNT[$i]=0
    if [ ${ALIVE} -gt 0 ]; then
      if [ ${STOP_COUNT[$i]} -eq 4 ]; then
        if [ ${WAIT_STOP[$i]} -eq 0 ]; then
          WrnLogMsg "Running instance detected: ${INAME}${INR}"
          LogMsg "Instance became automatically restarted: ${INAME}${INR}"
          #
          # the directory might exist if SAP instance has profile parameter "Autostart = 1"
          # because sapcontrol might not inform the cluster API about the instance startup
          # in this case
          #
          rm -r -f ${APIDIR}/sgesap_${SAPSYSTEMNAME}_${INAME}${INR}_stopped 2>&1 >/dev/null 
          STOP_COUNT[$i]=1
        else
          if [ ${WAIT_STOP[$i]} -ne 2 ]; then 
            WrnLogMsg "Running instance detected: ${INAME}${INR}"
            WrnLogMsg "Make sure instance gets stopped - monitoring for instance is inactive"
            WAIT_STOP[$i]=2
          fi
        fi
      fi
    fi
  fi

}

########################################################################
# Maintain monitor internal state: RED, YELLOW, GREEN. If RED then exit
# monitor to signal Serviceguard that the package has failed
#
# Parameters:
#
# $1 INSTNAME
# $2 INSTNR
# $3 ALIVE: FLAG if instanace is alive (1=yes, 0=no)
# $4 i: array index for all instances in a SG package
# $5 RETRY_COUNT: how many YELLOWS allowd before it turns into RED
#
########################################################################

function m_set_internal_states
{
  typeset fname=m_set_internal_states
  [ ${DBG} -gt 1 ] && BLogMsg $*

  typeset INAME=$1
  typeset INR=$2
  typeset ALIVE=$3
  typeset i=$4
  typeset RETRY_COUNT=$5

  RETRY_COUNT=${RETRY_COUNT:-${SGeSAP_RETRY_COUNT}}

  #
  # Figure out GREEN/YELLOW/RED
  #
  if [ ${ALIVE} -eq 0 ]; then
    # process not found - exit directly
    STATUS_FLAG[$i]=RED
    ErrLogMsg "Monitor reports ${INAME}${INR} - monitor state:${STATUS_FLAG[$i]}"
    # TODO ENHANCEMENT
    # TODO postpone exit until conditions have been evaluated (to allow instance restarts) 
    guard_exit_in_maint ${ERROR} ; if [ $? -eq 1 ]; then return 1; fi

  elif [ ${ALIVE} -eq 1 ]; then
    STATUS_FLAG[$i]=YELLOW
    (( YELLOW_COUNT[$i] = ${YELLOW_COUNT[$i]} + 1 ))
    if [ ${YELLOW_COUNT[$i]} -ge ${RETRY_COUNT} ]; then
      STATUS_FLAG[$i]=RED
      ErrLogMsg "Monitor reports ${INAME}${INR} - monitor state:${STATUS_FLAG[$i]}"
      guard_exit_in_maint ${ERROR} ; if [ $? -eq 1 ]; then return 1; fi
    else
      WrnLogMsg "Monitor reports ${INAME}${INR} - monitor state:${STATUS_FLAG[$i]},${YELLOW_COUNT[$i]}"
    fi

  elif [ ${ALIVE} -eq 2 ]; then # "fully" alive 
    if [ "X${STATUS_FLAG[$i]}" != "XGREEN" ]; then  # let it also recover from RED (prev. YELLOW only)
      # mon_reset  # can't be done here, because it'll reset all flags
      STATUS_FLAG[$i]=GREEN; YELLOW_COUNT[$i]=0
      LogMsg "Monitor reports ${INAME}${INR} recovered - state:${STATUS_FLAG[$i]}"
    else
      DbgLogMsg "Monitor reports ${INAME[$i]}${INR[$i]} - running"
    fi

  else
      ErrLogMsg "Unhandled value for ALIVE: ${ALIVE}"
  fi
  return 0
}

########################################################################
# Maintain monitor internal state: RED, YELLOW, GREEN. If RED then exit
# monitor to signal Serviceguard that the package has failed
#
# Parameters:
#
# $1 INSTNAME
# $2 INSTNR
# $3 ALIVE: FLAG if instanace is alive (1=yes, 0=no)
# $4 i: array index for all instances in a SG package
#
########################################################################

function m_set_alerts
{
  typeset fname=m_set_alerts

  [ ${DBG} -gt 1 ] && BLogMsg $*

  typeset INAME=$1
  typeset INR=$2
  typeset ALIVE=$3
  typeset i=$4

  if [ "X${STATUS_FLAG[$i]}" = "XYELLOW" ]; then
    alert_params_new[${PKG_SGESAP_SERVICE_YELLOW}]="${alert_params_new[${PKG_SGESAP_SERVICE_YELLOW}]}${INAME[$i]}${INR[$i]},"
    (( alert_status_new = ${alert_status_new} | ${PKG_SGESAP_SERVICE_YELLOW} ))
  elif [ "X${STATUS_FLAG[$i]}" = "XRED" ]; then
    alert_params_new[${PKG_SGESAP_SERVICE_RED}]="${alert_params_new[${PKG_SGESAP_SERVICE_RED}]}${INAME[$i]}${INR[$i]},"
    (( alert_status_new = ${alert_status_new} | ${PKG_SGESAP_SERVICE_RED} ))
  fi

}
########################################################################
# handle alert for all instances
#
# Parameters:
#
#
########################################################################

function m_handle_alerts
{
  typeset fname=m_handle_alerts
  [ ${DBG} -gt 1 ] && BLogMsg $*

  typeset -i i

    #
    # Handle alerts
    # - only if ALERT_STAGE_FILE is set
    #
    
    #ALERT_STAGE_FILE="" # uncomment to switch of alert file handling temporarily
    
    if [ -n "${ALERT_STAGE_FILE}" ]; then    
      touch ${ALERT_STAGE_FILE} # monitor alert heartbeat signal
      
      i=1
      while [ ${i} -le ${alert_status_new} ]; do
        (( bittest = ${alert_status_new} / ${i} % 2 ))
        if (( alert_status_new & i )); then
          if [ "X${alert_params_new[$i]}" != "X${ALERT_PARAMS[$i]}" ]; then
            alert_changed=1
            break
          fi
        fi
        (( i = ${i} * 2 ))
      done
      
      if [ "X${alert_status_new}" != "X${ALERT_STATUS}" -o ${alert_changed} -eq 1 ]; then
        LogMsg2 2 "Alerts changed (${alert_status_new},${alert_changed},${i})"
        :>${ALERT_STAGE_FILE}
        i=1
        while [ ${i} -le ${alert_status_new} ]; do
          (( bittest = ${alert_status_new} / ${i} % 2 ))
          if (( alert_status_new & i )); then
            echo package:${SG_PACKAGE}'|'sgesap_alert:${ALERT_TEXT[$i]}:${i}'|'name=${ALERT_TEXT[$i]}:${i} >> ${ALERT_STAGE_FILE}
            echo package:${SG_PACKAGE}'|'sgesap_alert:${ALERT_TEXT[$i]}:${i}'|'alert_parameter=${alert_params_new[${i}]%,}  >> ${ALERT_STAGE_FILE}
            LogMsg2 2 "SGeSAP monitor alert raised: ${ALERT_TEXT[$i]} ${PKG_ALERT_PARAMS[${i}]}"
            ALERT_PARAMS[$i]=${alert_params_new[$i]}
          fi
         (( i = ${i} * 2 ))
        done
        #
        # approve alerts to get picked up by sapgetconf
        #
        get_lock ${ALERT_TEMP_FILE}
        if [ $? -eq 0 ]; then
          cp ${ALERT_STAGE_FILE} ${ALERT_TEMP_FILE}
          ALERT_STATUS=${alert_status_new}
          free_lock ${ALERT_TEMP_FILE}
        fi
      fi
    fi
    # end of handling alerts    

} 
