#"@(#) SGeSAP/LX - 12.10.00 - $Rev: 1651 $ - Last modified 02/02/17"
#
# **********************************************************************
# *                                                                    *
# * HPE SGeSAP/LX                                                      *
# *                                                                    *
# * SAP Netweaver module dispatcher script                             *
# *                                                                    *
# * (c) Copyright 1996-2015 Hewlett-Packard Co.                        *
# * (c) Copyright 2015-2017 Hewlett Packard Enterprise Co.             *
# *                                                                    *
# *                                                                    *
# * Description: Serviceguard Extension for SAP control logic          *
# *                                                                    *
# * This control script file must not be edited !                      *
# * Any changes made to it will be overwritten when you upgrade to the *
# * next revision of HPE SGeSAP.                                       *
# *                                                                    *
# * Changing this file may lead to unrecoverable package startup       *
# * and/or shutdown problems.                                          *
# *                                                                    *
# **********************************************************************


. ${SGCONF}/scripts/sgesap/sap_control.sh

set -m
trap "wait;sg_log 2 \"Interrupting SGeSAP ${script_name} '${op_code}' runtime steps ...\"; exit 2" 2

case ${op_code} in
  start)

    check_versions
    cross_check
    init_params_sap_inst
    check_debug
    init_cdir

    # test_proc cmenqord
    # if [ $? -eq 0 ]; then
    #   if [ ${TPID} != 0 ]; then
    #     kill -SIGUSR1 ${TPID}
    #   fi
    # fi

    detect_module "sgesap/sapinfra"
    if [ $? -eq 1 ]; then
      detect_module "sgesap/sapextinstance"
      if [ $? -eq 1 ]; then
      detect_module "sgesap/dbinstance"
        if [ $? -eq 1 ]; then
          dns_sync
        fi
      fi
    fi

    [ ${JCI_T} ] && start_cs ${JCINAME} ${JCINR} ${JCIRELOC}
    [ ${CI_T} ]  && sync_db CI && start_cs ${CINAME} ${CINR} ${CIRELOC}
    [ ${RE_T} ]  && start_cs ${REPNAME} ${REPNR} ${REPRELOC}
    [ ${RE2_T} ] && start_cs ${REPNAME2} ${REPNR2} ${REPRELOC2}
    wait

    [ ${D_T}${JD_T} ] && sync_db &&
    { 
      [ ${D_T} ] && start_d
      [ ${JD_T} ] && start_jd
    }
    wait

    [ ${MDM_T} ] && sync_db MDM && start_mdm
    [ ${WD_T} ] && start_wd
    [ ${GW_T} ] && start_gw
  ;;
  stop)

    check_versions
    cross_check
    init_params_sap_inst
    init_cdir

    #
    # skip the standard shutdown
    # for the case of a cross-subnet
    # emergency

    if [ $? -eq 0 ]; then
      [ ${JD_T} ]  && stop_jd
      [ ${D_T} ]   && stop_d
      wait
      
      [ ${GW_T} ] && stop_gw
      [ ${WD_T} ] && stop_wd
      [ ${MDM_T} ] && stop_mdm
      wait

      [ ${CI_T} ]  && stop_cs ${CINAME} ${CINR} ${CIRELOC}
      [ ${JCI_T} ] && stop_cs ${JCINAME} ${JCINR} ${JCIRELOC}
      [ ${RE_T} ]  && stop_cs ${REPNAME} ${REPNR} ${REPRELOC}
      [ ${RE2_T} ] && stop_cs ${REPNAME2} ${REPNR2} ${REPRELOC2}
      wait
    fi
  ;;
  validate)

    check_versions

    SGeSAP_RETRY_COUNT=1
    init_params_sap_inst

    static_param_latch SAP_SYSTEM
    static_param_latch SAP_INSTANCE
    static_param_latch SAP_REPLICATED_INSTANCE
    static_param_latch SAP_VIRTUAL_HOSTNAME
    static_param_latch SAPCONTROL_USAGE

    static_param_latch MDM_MDSHOSTSPEC
    static_param_latch MDM_CREDENTIALS
    static_param_latch MDM_REPOSITORYSPEC
    
    SAPVERIFY="${SGSBIN}/sapverify"
    if [ -x ${SAPVERIFY} ]; then
       ${SAPVERIFY} -p "${SG_PACKAGE}" -sg_op_code "${op_code}" -call "$0"
       rc=$?
       [ ${rc} -gt 0 ] && exit ${rc}
    else
        sg_log 0 "WARNING: Couldn't find or execute ${SAPVERIFY}"
    fi
  ;;
  reconfigure_start)
  ;;
  reconfigure_stop)
  ;;
  *)
    sg_log 0 "ERROR: Runtime called with illegal package mode: '${op_code}'"
    exit 2
  ;;
esac

sg_log 2 "Leaving SGeSAP ${script_name} '${op_code}' runtime steps ..."

