"""
@(#) SGeSAP/LX - 12.10.00 - $Rev: 1714 $ - Last modified 03/03/17

SAP HANA database HA provider
Please add lines similar to this to your global.ini:

CONTRIB - NOT FOR PRODUCTION USE!

SLES:
[ha_dr_provider_SGeSAPcl]
provider = SGeSAPcl
path = /opt/cmcluster/lib
execution_order = 50

REDHAT:
[ha_dr_provider_SGeSAPcl]
provider = SGeSAPcl
path = /usr/local/cmcluster/lib
execution_order = 50


(C) Copyright 2016-17 Hewlett Packard Enterprise Development Company, L.P.

"""

from hdb_ha_dr.client import HADRBase, Helper
from datetime import timedelta,datetime
from tempfile import mkstemp
#from matplotlib.dates import date2num
import traceback, os, time, re, threading, errno
import sys, getopt

class HADRReplicationConfiguration():
    """system_replication section of global.ini"""
    _replConfig = None
    _params = {}
    _name = None

    def __init__(self, name):
        self._name = name
        import ConfigMgrPy
        self._replConfig = ConfigMgrPy.LayeredConfiguration('global.ini', ConfigMgrPy.CUSTOMER)
        self.reload()

    def reload(self):
        self._params.clear()
        self._replConfig.reload()
        tmpParams = self._replConfig.getValues('system_replication')

        for (k, v) in tmpParams:
            self._params[k] = v

    def hasKey(self, key):
        return self._params.has_key(key)

    def get(self, key):
        return self._params.get(key)

    def toString(self):
        return str(self._params)


class SGeSAPcl(HADRBase):

    apiVersion = 1

    # nullf=open(os.devnull,'w')

    # initialization parameters
    mySid = ''
    myPkgName = ''
    myPkgSafesyncRes = ''
    myPkgStatus = ''
    multinode=False
    RepStatusTexts = [ 'to be unconfigured', 'a replication error', 'an unknown state', 'initialization activity', 'synchronization activity', 'to be active' ]

    # for deactivation of block types
    PERSISTENCE_BLOCK, DISABLEMENT_BLOCK, ELIGIBILITY_BLOCK = '','',''
    # PERSISTENCE_BLOCK = 'skip'
    # DISABLEMENT_BLOCK = 'skip'
    # ELIGIBILITY_BLOCK = 'skip'

    # timeout parameter tuning section
    RETRY_COUNT = 5
    NFS_DELAY = 7
    RESOURCE_DELAY = 10
    DISABLE_DELAY = 15
    PIPE_DELAY = 10

    # internal book-keeping of safesync state
    blocks = 'in initialization' # fuzzy logic...

    # HSR replication states
    SAP_HANA_REPLICATION_STATUS_NOHSR=10
    SAP_HANA_REPLICATION_STATUS_ERROR=11
    SAP_HANA_REPLICATION_STATUS_UNKNOWN=12
    SAP_HANA_REPLICATION_STATUS_INITIALIZING=13
    SAP_HANA_REPLICATION_STATUS_SYNCING=14
    SAP_HANA_REPLICATION_STATUS_ACTIVE=15

    def __init__(self, *args, **kwargs):
        super(SGeSAPcl, self).__init__(*args, **kwargs)

        if __name__ == '__main__':
            self.tracer=DBGTracer(4)

        self.tracer.debug('enter init')

       # initialize variables
        self.mySid = os.environ['SAPSYSTEMNAME']
        self.dirInstance = os.environ['DIR_INSTANCE']
        if not self.mySid:
            self.tracer.error('SAPSYSTEMNAME not set - execute as sidadm')
        if not self.dirInstance:
            self.tracer.error('DIR_INSTANCE not set - execute as sidadm')
        self.tracer.debug('sid: '+self.mySid)
        self.tracer.debug('dirInstance: '+self.dirInstance)
        self.myPkgSafesyncRes = 'safesync'+self.mySid
        configfile = open('/etc/cmcluster.conf','r').read()
        ini = configfile.find('SGCONF')+7
        rest = configfile[ini:]
        search_enter = rest.find('\n')
        self.sgconf = rest[:search_enter]+'/'
        self.tracer.debug('sgconf: '+self.sgconf)
        ini = configfile.find('SGSBIN')+7
        rest = configfile[ini:]
        search_enter = rest.find('\n')
        self.sgrun = rest[:search_enter]+'/'
        self.tracer.debug('sgrun: '+self.sgrun)
        self.lastTimestamp = self.convertSapTime("1970-01-01T00:00:00.000000Z")

        # read global.ini
        self.repConfig = HADRReplicationConfiguration('replication')
        if self.config.hasKey('retry_count'):
            self.RETRY_COUNT = self.config.get('retry_count')
            self.tracer.info('retry_count: '+self.RETRY_COUNT)
        if self.config.hasKey('nfs_delay'):
            self.NFS_DELAY = self.config.get('nfs_delay')
            self.tracer.info('nfs_delay: '+self.NFS_DELAY)
        if self.config.hasKey('pipe_delay'):
            self.PIPE_DELAY = self.config.get('pipe_delay')
            self.tracer.info('pipe_delay: '+self.PIPE_DELAY)
        if self.config.hasKey('resource_delay'):
            self.RESOURCE_DELAY = self.config.get('resource_delay')
            self.tracer.info('resource_delay: '+self.RESOURCE_DELAY)
        if self.config.hasKey('persistence_block'):
            self.PERSISTENCE_BLOCK = self.config.get('persistence_block')
        if self.config.hasKey('disablement_block'):
            self.DISABLEMENT_BLOCK = self.config.get('disablement_block')
        if self.config.hasKey('eligibility_block'):
            self.ELIGIBILITY_BLOCK = self.config.get('eligibility_block')
        if self.PERSISTENCE_BLOCK == 'skip':
            self.tracer.warning('persistence block handling is configured to be skipped')
        if self.DISABLEMENT_BLOCK == 'skip':
            self.tracer.warning('disablement block handling is configured to be skipped')
        if self.ELIGIBILITY_BLOCK == 'skip':
            self.tracer.warning('eligibility block handling is configured to be skipped')

        # we need a semaphore to avoid overlapping execution
        # if HANA changes state of several services at (almost)
        # the same time
        self.pipeLock = threading.Lock()


        # identify locally running package
        try:
            if not self.sgGetPackageInfo():
                return 1
        finally:
            self.tracer.debug('leave init')

    def __del__(self):
        self.tracer.debug('enter del')
        self.tracer.debug('leave del')

    def about(self):
        self.tracer.debug('enter about')
        self.tracer.debug('leave about')
        return {'provider_company'     :'HPE',
                'provider_description' :'Serviceguard (non-production, contributed functionality)',
                'provider_version'     :'12.10.00' }

    def startup(self, hostname, storage_partition, system_replication_mode, **kwargs):
        self.tracer.debug('enter startup hook; %s' % locals())
        self.tracer.debug(self.config.toString())

        self.tracer.info('leave startup hook')
        return 0

    def shutdown(self, hostname, storage_partition, system_replication_mode, **kwargs):
        self.tracer.debug('enter shutdown hook; %s' % locals())
        self.tracer.debug(self.config.toString())

        self.tracer.info('leave shutdown hook')
        return 0

    def failover(self, hostname, storage_partition, system_replication_mode, **kwargs):
        self.tracer.debug('enter failover hook; %s' % locals())
        self.tracer.debug(self.config.toString())

        self.tracer.info('leave failover hook')
        return 0

    def stonith(self, failingHost, **kwargs):
        self.tracer.debug("enteleave stonith hook; %s" % locals())
        self.tracer.debug(self.config.toString())

        # e.g. stonith of params["failed_host"]
        # e-g- set vIP active

        self.tracer.info('leave stonith hook')
        return 0

    def preTakeover(self, isForce, **kwargs):
        """Pre takeover hook."""
        self.tracer.info('%s.preTakeover method called with isForce=%s' % (self.__class__.__name__, isForce))
        self.repConfig.reload()
        if self.repConfig.hasKey('operation_mode'):
            self.tracer.info('Configured operation mode found')
            if self.opModeFlag:
                opMode = self.repConfig.get('operation_mode')
                self.tracer.info('Saving as last seen operation mode: '+opMode)
                try:
                    opModeFd = open(self.opModeFlag,'w')
                except IOError, ioex:
                    self.tracer.error('IO Error')
                opModeFd.write(opMode)
                opModeFd.close()

        # variables used here for saving global.ini's actual_mode:
        # "actualmode.tmp"  - actual name of flag file
        # actualmodeFlag    - path to flag file: /usr/sap/<SID>/<instance>/work/.sgesap/api/actualmode.tmp
        #                   - is actually in /hana/shared/<SID>/<instance>/work/.sgesap/api/actualmode.tmp
        # actualmodeFd      - file descriptor used for writing/reading
        # actual_mode       - current value of actual_mode
        if self.repConfig.hasKey('actual_mode'):
            self.tracer.info('[system_replication]actual_mode found')
            if self.actualmodeFlag:
                actual_mode = self.repConfig.get('actual_mode')
                self.tracer.info('Saving as last seen [system_replication]actual_mode: '+actual_mode)
                try:
                    actualmodeFd = open(self.actualmodeFlag,'w')
                except IOError, ioex:
                    self.tracer.error('IO Error')
                actualmodeFd.write(actual_mode)
                actualmodeFd.close()

        if self.config.hasKey('retry_count'):
            self.RETRY_COUNT = self.config.get('retry_count')
            self.tracer.info('retry_count: '+self.RETRY_COUNT)
        else:
            self.RETRY_COUNT = 5

        if not isForce:
            if not self.sgGetPackageInfo():
                return 0
            elif self.myPkgStatus != 's':
                count = 0
                (c,o) = self.runOsCommand("%scmviewcl -f line -l cluster" % self.sgrun)
                if c == 0:
                    for line in str.split(o):
                        if re.match('status=down', line) or re.match('state=stable', line):
                            count += 1
                        # self.tracer.debug(line)
                self.tracer.debug('rejection conditions eval: downStable %d, maintenanceMode %s' % (count, self.maintenanceMode))
                if count == 2:
                    self.tracer.warning('manual takeover while cluster is halted')
                    return 0
                elif self.maintenanceMode:
                    self.tracer.warning('manual takeover in primary package maintenance mode')
                    return 0
                else:
                    self.tracer.error('cluster rejects manual takeover attempt')
                    self.tracer.info('stop cluster or put primary package in maintenance mode to override')
                    # TODO: depends on future SAP enhancement:
                    # self.tracer.info('use hdbnsutil sr_takeover --force to override at your own risk')
                    return 1
        else:
            # possible force-takeover only code
            # usually nothing to do here
            self.tracer.warning("forced takeover - safesync gate was bypassed")
        return 0

    def postTakeover(self, rc, **kwargs):
        """Post takeover hook."""
        self.tracer.info("%s.postTakeover method called with rc=%s" % (self.__class__.__name__, rc))

        if rc == 0:
            # normal takeover succeeded
            # the following might be necessary if HANA wouldn't restart during takeover
            # self.sgGetPackageInfo() # might be first time for a secondary
            # if self.myPkgStatus == 's':
            #     open(self.takeoverFlag, 'a').close()
            #     os.utime(self.takeoverFlag, None)
            #     self.tracer.info('takeover token received')
            #     self.tracer.info('trying to activate persistence and eligibility safesync blocks in reaction to takeover...')
            #     self.tracer.info('acquiring safesync hook lock...')
            #     self.pipeLock.acquire()
            #     self.sgPBlockOn()
            #     self.sgEBlockOn()
            #     self.blocks = "active"
            #     self.pipeLock.release()
            #     self.tracer.info('released safesync hook lock...')
            return 0
        elif rc == 1:
            # waiting for force takeover
            return 0
        elif rc == 2:
            # error, something went wrong
            return 0

    def srConnectionChanged(self, parameters, **kwargs):
        self.tracer.debug("enter srConnectionChanged hook; %s" % locals())

        # Report the event to the log(s)
        hostname = parameters['hostname']
        port = parameters['port']
        database = parameters['database']
        databaseStatus = parameters['database_status']
        systemStatus = parameters['system_status']
        status = parameters['status']
        timestamp = self.convertSapTime(parameters['timestamp'])
        is_in_sync = parameters['is_in_sync']

        if database:
            tenantText = ' for database '+database
        else:
            tenantText = ''
        if hostname:
            hostnameText = ' of host '+hostname
        else:
            hostnameText = ''
        try:
            statusText = self.RepStatusTexts[int(status)-10]
        except IndexError:
            statusText = 'an illegal return code'
        self.sgLog('replication of HANA service'+tenantText+' on port '+str(port)+hostnameText+' reports '+statusText+' ('+str(status)+')', True)

        # don't even try to block the lock, if the SAP event does not require a cluster event
        # if self.isObsolete(timestamp, systemStatus, is_in_sync):
        #     return 0

        self.tracer.info('acquiring safesync hook lock...')
        if not self.pipeLock.acquire(False):
            self.tracer.warning('failed to acquire safesync lock - another safesync operation is in progress - retry later')
            return 1
        else:
            rc = 0
            try:
                # don't go any further, if the SAP event does not require a cluster event
                if self.isObsolete(timestamp, systemStatus, is_in_sync):
                    return 0

                if not self.sgGetPackageInfo():
                    return 1

                # prevent that HANA gets blocked if cluster is not fully operational
                # if self.maintenanceMode == True:
                #   self.tracer.warning('ignoring connection change event due to primary package maintenance mode...')
                    return 0
                if self.myPkgStatus == 'd':
                    self.tracer.error('rejecting connection change event due to primary package being inactive...')
                    self.tracer.warning('safesync block unreliable')
                    return 0 # return 1

                # propagate event into cluster package log
                # self.sgLog(str(parameters), False)

                # the safesync operation
                # if parameters['system_status'] == self.SAP_HANA_REPLICATION_STATUS_ACTIVE:
                if is_in_sync:
                    rc = self.sgUnblockPkg(parameters)
                else:
                    rc = self.sgBlockPkg(parameters)
            finally:
                self.pipeLock.release()
                self.tracer.info('released safesync hook lock...')

        self.tracer.debug('leave srConnectionChanged hook')
        return int(1 - rc)

    def isObsolete(self, timestamp, systemStatus, is_in_sync):
        if not is_in_sync:
            if self.blocks != "active":
                if self.lastTimestamp >= timestamp:
                    self.tracer.warning('discarded an out-of-sequence srConnectionChange event')
                else:
                   return False
        elif systemStatus == self.SAP_HANA_REPLICATION_STATUS_ACTIVE:
            if self.blocks != "inactive":  # it could be false or undefined
                if self.lastTimestamp > timestamp:
                    self.tracer.warning('discarded an out-of-sequence srConnectionChange event')
                else:
                    return False
        self.tracer.info('overall safesync block remains '+self.blocks)
        return True

    # ME: move block subroutines into own class sgAutoHandler

    def sgPipeWrite(self,code,delay):
        for _ in range(int(self.RETRY_COUNT)):
            try:
                providerPipeFd = open(self.providerPipe,'w')
            except IOError, ioex:
                self.tracer.error('IO: ha_dr_provider pipe cannot be accessed with code '+code+' ('+os.strerror(ioex.errno)+')')
                return False
            try:
                os.remove(self.successFlag)
            except Exception:
                pass
            try:
                os.remove(self.errorFlag)
            except Exception:
                pass
            providerPipeFd.write(code)
            providerPipeFd.close()
            if self.isSuccessful(delay):
                return True
            self.tracer.warning('ha_dr_provider pipe timeout - retry later')
            time.sleep(self.PIPE_DELAY)
        return False

    def sgLog(self,code,traceFlag):
        # self.sgPipeWrite(':', 1)
        # self.sgPipeWrite(str(code), 1)
        if (traceFlag == True):
            self.tracer.info(str(code))
        return 0

    def sgStickyOn(self):
        if self.DISABLEMENT_BLOCK == 'skip':
            return True
        if not self.sgPipeWrite('S', self.NFS_DELAY):
            self.tracer.warning('stickyness could not be propagated to all nodes')
            open(self.stickyFlag, 'a').close()
            os.utime(self.stickyFlag, None)
            try:
                os.chmod(self.stickyFlag, 0774)
            except Exception:
                pass
            return False
        return True

    def sgStickyOff(self):
        if self.DISABLEMENT_BLOCK == 'skip':
            return True
        if not self.sgPipeWrite('R', self.NFS_DELAY):
            self.tracer.warning('stickyness could not be removed from all nodes')
            try:
                os.remove(self.stickyFlag)
            except Exception:
                pass
            return False
        return True

    def sgPBlockOn(self):
        if self.PERSISTENCE_BLOCK == 'skip':
            return True
        if self.sgPipeWrite('B', self.NFS_DELAY):
            self.tracer.info('success: safesync persistence block is active now')
            self.tracer.debug('propagated flag file')
            return True
        if [ self.blocks == "inactive" ]:
            self.tracer.warning('safesync persistence block could not be set on all nodes')
        open(self.safeSyncFlag, 'a').close()
        os.utime(self.safeSyncFlag, None)
        try:
            os.chmod(self.safeSyncFlag, 0774)
        except Exception:
            pass
        open(self.takeoverFlag, 'a').close()
        os.utime(self.takeoverFlag, None)
        try:
            os.chmod(self.takeoverFlag, 0774)
        except Exception:
            pass
        return False

    def sgPBlockOff(self):
        if self.PERSISTENCE_BLOCK == 'skip':
            return True
        if self.sgPipeWrite('U', self.NFS_DELAY):
            self.tracer.info('success: safesync persistence block is inactive now')
            self.tracer.debug('removed block flag file')
            try:
                os.remove(self.takeoverFlag)
            except Exception:
                pass
            return True
        if [ self.blocks == "active" ]:
            self.tracer.warning('safesync persistence block could not be removed from all nodes')
        try:
            os.remove(self.takeoverFlag)
        except Exception:
            pass
        try:
            os.remove(self.safeSyncFlag)
        except Exception:
            pass
        return False

    def sgEBlockOn(self):
        if self.ELIGIBILITY_BLOCK == 'skip':
            return True
        if self.sgPipeWrite('F', self.RESOURCE_DELAY):
            self.tracer.info('success: safesync eligibility block is active now')
            self.tracer.debug('generic resource is set to false on available remote nodes')
            return True
        self.tracer.warning('safesync eligibility block could not be set on all nodes')
        return False

    def sgEBlockOff(self):
        if self.ELIGIBILITY_BLOCK == 'skip':
            return True
        if self.sgPipeWrite('T', self.RESOURCE_DELAY):
            self.tracer.info('success: safesync eligibility block is inactive now')
            self.tracer.debug('generic resource is set to true on available remote nodes')
            return True
        self.tracer.warning('safesync eligibility block could not be removed from all nodes')
        return False

    def sgDBlockOn(self):
        if not self.isDisabled():
            self.sgStickyOff()
            if self.myPkgStatus == 'u':
                self.tracer.info('try to disable autorun for '+self.myPkgName)
                if self.isD1failureInjected():
                    self.tracer.warning('--- FOUND AN INJECTED PKG DISABLEMENT FAILURE ---')
                    return False
                self.sgPipeWrite('D', self.DISABLE_DELAY)
                if self.isDisabled():
                    self.tracer.info('success: safesync disablement block is active now')
                    return True
                self.tracer.error('failed to deactivate autorun')
                return False
            # we cannot disable during package startup, that's not an error
            elif self.myPkgStatus != 's':
                self.tracer.warning('package status does not allow autorun disablement')
                return False
            return True
        else: # already disabled
            self.tracer.info('success: safesync disablement block is active now')
            self.tracer.warning('autorun won\'t become enabled automatically after secondary syncup')
            self.sgStickyOn()
            return True

    def sgDBlockOff(self):
        if self.isDisabled():
            if not self.isSticky():
                if self.myPkgStatus == 'u':
                    if self.isD2failureInjected(): # for QA and debugging purposes
                        self.tracer.warning('--- FOUND AN INJECTED PKG ENABLEMENT FAILURE ---')
                        return False
                    self.sgPipeWrite('E', self.DISABLE_DELAY)
                    if not self.isDisabled():
                        self.tracer.info('success: safesync disablement block is inactive now')
                        return True
                    self.tracer.error('failed to activate autorun')
                    return False
                else:
                    self.tracer.info('package status does not allow autorun enablement')
                    return False
            else:
                self.tracer.info('success: safesync disablement block is inactive now')
                self.tracer.warning('autorun remains manually disabled')
                self.sgStickyOff()
                return True
        else: # already enabled
            self.tracer.info('autorun is already enabled')
            self.sgStickyOff()
            return True

    def isDisabled(self):
        self.tracer.debug('myPkgName: '+self.myPkgName)
        (c,o) = self.runOsCommand("%scmviewcl -f line -p %s" % (self.sgrun, self.myPkgName))
        if c == 0:
            for line in str.split(o):
                if re.match("^autorun=disabled$", line):
                    return True
        return False

    def isBlocked(self):
        return os.path.isfile(self.safeSyncFlag)

    def isSticky(self):
        return os.path.isfile(self.stickyFlag)

    def isSuccessful(self, delay):
        time.sleep(1)
        for _ in range(int(self.RETRY_COUNT)):
            if os.path.isfile(self.successFlag):
                return True
            if os.path.isfile(self.errorFlag):
                return False
            time.sleep(delay)
        return False

    def isD1failureInjected(self):
        return os.path.isfile(self.injectionD1failureFlag)

    def isD2failureInjected(self):
        return os.path.isfile(self.injectionD2failureFlag)

    def sgBlockPkg(self, parameters):
        self.tracer.debug('enter sgBlockPkg')
        timestamp = parameters['timestamp']

        # Create all failover preventions
        # 1. Add block on filesystem
        self.tracer.debug('handle persistence block...')
        rc = self.sgPBlockOn()
        # 2. Set generic resource to false
        self.tracer.debug('handle eligibility block...')
        rc = rc and self.sgEBlockOn()
        # 3. Disable package, but only if it is not already disabled
        if self.multinode == False:
            self.tracer.debug('handle disablement block...')
            rc = rc and self.sgDBlockOn()
        # adjust internal state
        if rc:
            self.blocks = "active"
            self.lastTimestamp = self.convertSapTime(timestamp)
            self.tracer.info('success: safesync blocks are active in reaction to change event on '+timestamp)
        else:
            self.blocks = "unknown"
        self.tracer.debug('leave sgBlockPkg')
        return rc

    def sgUnblockPkg(self, parameters):
        self.tracer.debug('enter sgUnblockPkg')
        timestamp = parameters['timestamp']

        # Remove all failover preventions
        # 1. Remove block from filesystem
        self.tracer.debug('handle persistence block...')
        rc = self.sgPBlockOff()
        # 2. Set generic resource to true
        self.tracer.debug('handle eligibility block...')
        rc = rc and self.sgEBlockOff()
        # 3. Enable package, but only if SGeSAP disabled it
        if self.multinode == False:
            self.tracer.debug('handle disablement block...')
            rc = rc and self.sgDBlockOff()
        if rc:
            self.blocks = "inactive"
            self.lastTimestamp = self.convertSapTime(timestamp)
            self.tracer.info('success: safesync blocks are inactive in reaction to change event on '+timestamp)
        else:
            self.blocks = "unknown"
        self.tracer.debug('leave sgUnblockPkg')
        return rc

    def sgGetPackageInfo(self):
        self.tracer.debug('enter sgGetPackageInfo')

        self.myPkgStatus, self.maintenanceMode = '', False # unknown, so far

        # if a primary package name is already known, try to use its name to determine status on this node
        if self.myPkgName:
            self.tracer.debug('enter fast cluster query')
            (c,o) = self.runOsCommand("%scmviewcl -f line -p %s" % (self.sgrun,self.myPkgName))
            if c == 0:
                for line in str.split(o):
                    # self.tracer.debug("cmviewcl dump:"+line)
                    if re.search('^node:'+os.uname()[1]+'\\|status=[ud][pe]', line):
                        self.myPkgStatus='u' # up
                    elif re.match('^node:'+os.uname()[1]+'\\|status=starting', line):
                        self.myPkgStatus='s' # starting
                    elif re.search('^node:'+os.uname()[1]+'\\|status=[hf]a[li][tl]i', line):
                        self.myPkgStatus='h' # halting
                    elif re.search('^node:'+os.uname()[1]+'\\|status=[drb][oel][wlo][noc]', line):
                        self.myPkgStatus='d' # down
                    elif re.search('^state=maintenance$', line):
                        self.maintenanceMode = True
            self.tracer.debug('passed fast cluster query')

        if self.myPkgStatus == '': # unknown, still

            # determine name and status of the primary package of this system on this node
            self.tracer.debug('enter comprehensive cluster query')
            nodeValidated, sidValidated, primaryValidated = False, False, False
            tmpPkgName, tmpPkgStatus, self.multinode = '','x', False
            (c,o) = self.runOsCommand("%scmviewcl -f line -l package -v " % self.sgrun)
            if c == 0:
                found = False
                for line in str.split(o):
                    if re.match('package:\w*\\|name=\w*', line):
                        if  nodeValidated and sidValidated and primaryValidated and tmpPkgStatus != 'x': # found the package
                            found = True
                            break
                        # reset gathered values for scan of next package
                        tmpPkgName = line.split('=',1)[1]
                        tmpPkgStatus = 'x'
                        sidValidated = False
                        primaryValidated = False
                        nodeValidated = False
                        self.tracer.debug('Check package '+tmpPkgName)
                    elif re.search('^package:\w*\\|node:'+os.uname()[1]+'\\|name='+os.uname()[1], line):
                        nodeValidated = True
                    elif re.search('^package:\w*\\|node:'+os.uname()[1]+'\\|status=[ud][pe]', line):
                        tmpPkgStatus = 'u'
                    elif re.match('^package:\w*\\|node:'+os.uname()[1]+'\\|status=starting', line):
                        tmpPkgStatus = 's'
                    elif re.search('^package:\w*\\|node:'+os.uname()[1]+'\\|status=[hf]a[li][tl]i', line):
                        tmpPkgStatus = 'h'
                    elif re.match('^package:\w*\\|node:'+os.uname()[1]+'\\|status=[drb][oel][wlo][noc]', line):
                        tmpPkgStatus = 'd'
                    elif re.search('\\|sgesap/hdb_global/hdb_system='+self.mySid, line):
                        sidValidated = True
                    elif re.search('module_name:sgesap/hdbprimary\\|module_name=sgesap/hdbprimary$', line):
                        primaryValidated = True
                    elif re.search('^package:'+tmpPkgName+'\\|state=maintenance$', line):
                        self.maintenanceMode = True
                    elif re.search('^package:'+tmpPkgName+'\\|type=multi_node$', line):
                        self.multinode = True
                        self.tracer.debug('part of a HANA scale-out configuration')
                if not found:
                    self.tracer.error('Cluster package configuration not found')
                    raise NameError

            self.tracer.debug('passed comprehensive cluster query')

            if nodeValidated and sidValidated and primaryValidated and tmpPkgStatus != 'x': # found the package
                self.myPkgName, self.myPkgStatus = tmpPkgName, tmpPkgStatus
                self.tracer.info('Fitting Serviceguard primary package: '+self.myPkgName)

            # set flag and pipe locations for the package
            if self.myPkgName:
                if self.multinode == True:
                    tdir = self.dirInstance+'/work/.sgesap'
                    prefix = ''
                else:
                    tdir = '/var/tmp/.sgesap'
                    prefix = self.myPkgName+'.sgesap'
                self.providerPipe = '/var/tmp/.sgesap/'+self.myPkgName+'.sgesap.hadrpipe.tmp'
                self.tracer.debug('providerPipe: '+self.providerPipe)
                self.safeSyncFlag = tdir+'/api/'+prefix+'.asynclck.tmp'
                self.tracer.debug('safesyncFlag: '+self.safeSyncFlag)
                self.stickyFlag = '/var/tmp/.sgesap/api/'+self.myPkgName+'.sgesap.sticky.tmp'
                self.successFlag = '/var/tmp/.sgesap/api/'+self.myPkgName+'.sgesap.succ.tmp'
                self.errorFlag = '/var/tmp/.sgesap/api/'+self.myPkgName+'.sgesap.err.tmp'
                self.injectionD1failureFlag = '/var/tmp/.sgesap/api/'+prefix+'.d1inject.tmp'
                self.injectionD2failureFlag = '/var/tmp/.sgesap/api/'+prefix+'.d2inject.tmp'
                self.takeoverFlag = tdir+'/api/'+prefix+'.takeover.tmp'
                self.opModeFlag = tdir+'/api/'+prefix+'.opmode.tmp'
                self.actualmodeFlag = tdir+'/api/'+prefix+'.actualmode.tmp'

        if not self.myPkgStatus: # unknown, still
            self.tracer.warning('not possible to query a fitting Serviceguard package configuration')
            self.myPkgName=''
            return 1
        else:
            self.tracer.info('primary package info: \''+self.myPkgStatus+'\'')

        self.tracer.debug('leave sgGetPackageInfo')
        return self.myPkgStatus

    def convertSapTime(self, time):
        # Examples: "2016-03-19T13:38:30.335123+00:01"
        #           "2016-03-19 13:38:30.335123Z"
        if time[-1] == 'Z':
            return datetime.strptime(time.replace('T',' ',1)[:26], "%Y-%m-%d %H:%M:%S.%f")
        else:
            return datetime.strptime(time.replace('T',' ',1)[:26], "%Y-%m-%d %H:%M:%S.%f")+timedelta(hours=int(time[-6:-3]),minutes=int(time[-6]+time[-2:]))

    def runOsCommand(self, command):

        self.tracer.debug ("run os command: "+command)
        try:
            tmpFileOut = mkstemp(suffix='.out', prefix='fc')
            tmpFileErr = mkstemp(suffix='.err', prefix='fc')
            retCode = os.system("%s > %s 2> %s" % (command, tmpFileOut[1], tmpFileErr[1]))
            with open("%s" % tmpFileOut[1], 'r') as file:
                output = file.read()
            with open("%s" % tmpFileErr[1], 'r') as file:
                errors = file.read()
            os.close(tmpFileOut[0])
            os.close(tmpFileErr[0])
            os.remove(tmpFileOut[1])
            os.remove(tmpFileErr[1])

            if retCode != 0:
                self.tracer.debug("os call return code: %s" % retCode)
                self.tracer.debug("os call stderr:      %s" % errors)
                return (retCode, "OS operation finished unsuccessfully: %s" % errors)
            return (retCode, output)
        except Exception as e:
            print e
            return (-1, 'Error while executing OS command')

class DBGTracer(object):
    """Additional tracer - only for QA tests."""

    def __init__(self, tracelevel):
        self.tracelevel = tracelevel

    def logmsg(self, *messages):
        """Log message"""
        lastchar = '\n'
        for message in messages:
            new = True
            for line in str(message).splitlines():
                if not new:
                    print ";",
                if line:
                    print line,
                    new = False
                    lastchar = str(line)[-1:]
                else:
                    new = True
        if not lastchar == '\n' and not lastchar.isspace():
            print

    def debug(self, *messages):
        """Debug trace"""
        if self.tracelevel >4:
            stk = traceback.extract_stack()
            self.logmsg(stk[-2][0], stk[-2][1], "DEBUG", *messages)

    def info(self, *messages):
        """Info trace"""
        if self.tracelevel >3:
            stk = traceback.extract_stack()
            self.logmsg(stk[-2][0], stk[-2][1], "INFO", *messages)

    def warning(self, *messages):
        """Warning trace"""
        if self.tracelevel > 1:
            stk = traceback.extract_stack()
            self.logmsg(stk[-2][0], stk[-2][1], "WARNING", *messages)

    def error(self, *messages):
        """Error trace"""
        stk = traceback.extract_stack()
        self.logmsg(stk[-1][0], stk[-1][1], "ERROR", *messages)

    def fatal(self, *messages):
        """Fatal error trace"""
        stk = traceback.extract_stack()
        self.logmsg(stk[-2][0], stk[-2][1], "FATAL", *messages)


def main(argv):

    try:
        opts, args = getopt.getopt(argv,"t",["testrun"])
    except getopt.GetoptError:
        print 'SGeSAPcl.py [-t|--testrun]'
        print 'The testrun manipulates existing Serviceguard packages.'
        print 'Do not execute on a production environment.'
        sys.exit(2)

    if len(sys.argv) >1:
        #
        # a test call sequence
        #
        # the test run should ultimately restore package enablement/disablement
        # to the setting that was in place prior to the run
        #
        sgesap = SGeSAPcl()
        sgesap.tracer.info(str(sgesap.about()))

        sgesap.sgGetPackageInfo()
        # sgesap.sgStickyOn()
        sgesap.preTakeover(False)
        # simulation initialization: a running system
        sgesap.tracer.info('*** SIMULATION PREP: set running system with inactive safesync blocks  ***')
        sgesap.srConnectionChanged({'hostname': 'hanahost',
                                    'port': 30003, 'database': '',
                                    'status': sgesap.SAP_HANA_REPLICATION_STATUS_ACTIVE,
                                    'database_status': sgesap.SAP_HANA_REPLICATION_STATUS_ACTIVE,
                                    'system_status': sgesap.SAP_HANA_REPLICATION_STATUS_ACTIVE,
                                    'timestamp': '2016-02-19T13:32:30.394685-01:00',
                                    'is_in_sync': 1})
        try:
            os.remove(sgesap.injectionD1failureFlag)
        except Exception:
            pass


        # first step: uncritical replication error
        sgesap.tracer.info('*** TESTRUN STEP 1: a simulated planned instance stop event ***')
        sgesap.srConnectionChanged({'hostname': 'hanahost',
                                    'port': 30003, 'database': '',
                                    'status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                    'database_status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                    'system_status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                    'timestamp': '2016-02-19T13:32:30.394686-01:00',
                                    'is_in_sync': 1})

        # second step: package disablement failure
        sgesap.tracer.info('*** TESTRUN STEP 2: critical replication error, package disablement failure (if triggered) ***')
        try:
            open(sgesap.injectionD1failureFlag, 'a').close()
            sgesap.srConnectionChanged({'hostname': 'hanahost',
                                        'port': 30001, 'database': '',
                                        'status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                        'database_status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                        'system_status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                        'timestamp': '2016-02-19 13:32:30.394685Z',
                                        'is_in_sync': 0})
        finally:
            os.remove(sgesap.injectionD1failureFlag)

        # third step: critical replication error
        sgesap.tracer.info('*** TESTRUN STEP 3: critical replication error ***')
        sgesap.srConnectionChanged({'hostname': '',
                                    'port': 30001, 'database': '',
                                    'status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                    'database_status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                    'system_status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                    'timestamp': '2016-02-19T13:32:31.394685+01:00',
                                    'is_in_sync': 0})



        # fourth step: tenant initialization, but package is already disabled
        sgesap.tracer.info('*** TESTRUN STEP 4: tenant initialization, but pkg already disabled ***')
        sgesap.srConnectionChanged({'hostname': '',
                                    'port': 30023, 'database': 'DB_A1',
                                    'status': sgesap.SAP_HANA_REPLICATION_STATUS_INITIALIZING,
                                    'database_status': sgesap.SAP_HANA_REPLICATION_STATUS_INITIALIZING,
                                    'system_status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                    'timestamp': '2016-02-19T13:34:32.394685+01:00',
                                    'is_in_sync': 0})

        # fifth step: insufficient replication repair
        sgesap.tracer.info('*** TESTRUN STEP 5: insufficient replication repair ***')
        sgesap.srConnectionChanged({'hostname': '',
                                    'port': 30001, 'database': '',
                                    'status': sgesap.SAP_HANA_REPLICATION_STATUS_ACTIVE,
                                    'database_status': sgesap.SAP_HANA_REPLICATION_STATUS_ACTIVE,
                                    'system_status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                    'timestamp': '2016-02-19T13:36:33.394685+01:00',
                                    'is_in_sync': 0})

        # sixth step: package enablement failure
        sgesap.tracer.info('*** TESTRUN STEP 6: package enablement failure ***')
        try:
            open(sgesap.injectionD2failureFlag, 'a').close()
            sgesap.srConnectionChanged({'hostname': '',
                                        'port': 30023, 'database': 'DB_A1',
                                        'status': sgesap.SAP_HANA_REPLICATION_STATUS_ACTIVE,
                                        'database_status': sgesap.SAP_HANA_REPLICATION_STATUS_ACTIVE,
                                        'system_status': sgesap.SAP_HANA_REPLICATION_STATUS_ACTIVE,
                                        'timestamp': '2016-02-19T13:38:34.394685+01:00',
                                        'is_in_sync': 1})
        finally:
            os.remove(sgesap.injectionD2failureFlag)

        # seventh step: full replication repair
        sgesap.tracer.info('*** TESTRUN STEP 7: full replication repair ***')
        sgesap.srConnectionChanged({'hostname': '',
                                    'port': 30023, 'database': 'DB_A1',
                                    'status': sgesap.SAP_HANA_REPLICATION_STATUS_ACTIVE,
                                    'database_status': sgesap.SAP_HANA_REPLICATION_STATUS_ACTIVE,
                                    'system_status': sgesap.SAP_HANA_REPLICATION_STATUS_ACTIVE,
                                    'timestamp': '2016-02-19T13:38:35.394685+12:34',
                                    'is_in_sync': 1})

        # eighth step: outdated request
        sgesap.tracer.info('*** TESTRUN STEP 8: reject outdated disablement request ***')
        sgesap.srConnectionChanged({'hostname': '',
                                    'port': 30023, 'database': 'DB_A1',
                                    'status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                    'database_status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                    'system_status': sgesap.SAP_HANA_REPLICATION_STATUS_ERROR,
                                    'timestamp': '2016-02-19T13:38:35.394685-00:01',
                                    'is_in_sync': 0})



if __name__ == '__main__':
    main(sys.argv[1:])
